import { useClickOutside, useNamespace } from '@ibiz-template/vue-util';
import { defineComponent, ref, watch } from 'vue';
import { IBizUpload, UploadFile } from '../ibiz-upload/ibiz-upload';

export const IBizGridFileUploadPopover = defineComponent({
  name: 'IBizGridFileUploadPopover',
  props: {
    uploadUrl: {
      type: String,
      required: true,
    },
    downloadUrl: {
      type: String,
      required: true,
    },
    value: {
      type: Array<{
        name: string;
        id: string;
        url: string;
      }>,
      required: true,
    },
  },
  emits: {
    close: (_result: { isModified: boolean; resultFiles: UploadFile[] }) =>
      true,
  },
  setup(props, { emit }) {
    const ns = useNamespace('grid-upload');
    const upload = ref();
    const componentRef = ref();
    const isModified = ref(false);
    const fileList = ref<UploadFile[]>([]);
    // 点击外部处理
    useClickOutside(componentRef, async _evt => {
      emit('close', {
        isModified: isModified.value,
        resultFiles: fileList.value,
      });
    });

    // 值响应式变更
    watch(
      () => props.value,
      newVal => {
        fileList.value = newVal.map(item => {
          return { ...item, status: 'finished', percentage: 100 };
        });
      },
      { immediate: true },
    );

    const onFinish = (files: UploadFile[]) => {
      fileList.value.push(...files);
      isModified.value = true;
    };

    const selectFile = () => {
      upload.value.selectFile();
    };

    const deleteFile = (file: UploadFile) => {
      const index = fileList.value.findIndex(item => item.id === file.id);
      fileList.value.splice(index, 1);
      isModified.value = true;
    };

    return {
      ns,
      componentRef,
      upload,
      fileList,
      onFinish,
      selectFile,
      deleteFile,
    };
  },
  render() {
    return (
      <div
        ref='componentRef'
        style={'width:300px;height:200px'}
        class={this.ns.b()}
      >
        <IBizUpload
          ref='upload'
          uploadUrl={this.uploadUrl}
          downloadUrl={this.downloadUrl}
          multiple={true}
          on-finish={this.onFinish}
        ></IBizUpload>
        <div class={this.ns.b('list')}>
          {this.fileList.map(file => {
            return (
              <div class={this.ns.be('list', 'item')}>
                <div class={this.ns.be('list', 'info')}>
                  <a
                    class={this.ns.be('list', 'file-name')}
                    href={file.url}
                    download={file.name}
                  >
                    {file.name}
                  </a>
                  <div class={this.ns.be('list', 'actions')}>
                    <i-icon
                      onClick={() => this.deleteFile(file)}
                      type='md-close'
                    />
                  </div>
                </div>
              </div>
            );
          })}
        </div>
        <i-button
          type='text'
          onClick={this.selectFile}
          class={this.ns.e('add-button')}
        >
          <i-icon type='md-add' />
          添加文件
        </i-button>
      </div>
    );
  },
});
